import '../../common/codable.dart';
import '../../common/userid.dart';
import '../../src/communitiesids.dart';
import '../communitiesentity.dart';
import '../pollstatus.dart';
import 'announcementsquery.dart';

class ActivitiesQuery extends Encodable {
  late CommunitiesIds ids;
  UserId? author;
  String? tag;
  String? searchTerm;
  int pollStatus = PollStatus.all;
  bool trending = false;
  List<String> labels = [];
  Map<String, String> properties = {};
  String? reactionGroup;
  List<String> reactions = [];
  List<UserId> mentions = [];

  ActivitiesQuery(CommunitiesIds ids) {
    this.ids = ids;
  }

  static ActivitiesQuery feedOf(UserId id) {
    return ActivitiesQuery(CommunitiesIds.user(id));
  }

  static ActivitiesQuery inTopic(String id) {
    return ActivitiesQuery(CommunitiesIds.topic(id));
  }

  static ActivitiesQuery inAllTopics() {
    return ActivitiesQuery(CommunitiesIds.topics([]));
  }

  static ActivitiesQuery inGroup(String id) {
    return ActivitiesQuery(CommunitiesIds.group(id));
  }

  static ActivitiesQuery inAllGroups() {
    return ActivitiesQuery(CommunitiesIds.groups([]));
  }

  static ActivitiesQuery timeline() {
    return ActivitiesQuery(CommunitiesIds.timeline());
  }

  static ActivitiesQuery everywhere() {
    return ActivitiesQuery(CommunitiesIds.everywhere());
  }

  static ActivitiesQuery commentsToActivity(String id) {
    return ActivitiesQuery(CommunitiesIds.activity(id));
  }

  static ActivitiesQuery bookmarkedActivities() {
    ActivitiesQuery query = ActivitiesQuery(CommunitiesIds.everywhere());
    query.reactionGroup = "bookmarks";
    return query;
  }

  static ActivitiesQuery reactedActivities(List<String>? reactions) {
    ActivitiesQuery query = ActivitiesQuery(CommunitiesIds.everywhere());
    query.reactionGroup = "reactions";
    if (reactions != null) {
      query.reactions = reactions;
    }
    return query;
  }

  static ActivitiesQuery votedActivities(List<String>? options) {
    ActivitiesQuery query = ActivitiesQuery(CommunitiesIds.everywhere());
    query.reactionGroup = "votes";
    if (options != null) {
      query.reactions = options;
    }
    return query;
  }

  ActivitiesQuery byUser(UserId id) {
    this.author = id;
    return this;
  }

  ActivitiesQuery withTag(String tag) {
    this.tag = tag;
    return this;
  }

  ActivitiesQuery withText(String searchTerm) {
    this.searchTerm = searchTerm;
    return this;
  }

  ActivitiesQuery withPollStatus(int status) {
    this.pollStatus = status;
    return this;
  }

  ActivitiesQuery onlyTrending(bool trending) {
    this.trending = trending;
    return this;
  }

  ActivitiesQuery withLabels(List<String> labels) {
    this.labels = labels;
    return this;
  }

  ActivitiesQuery withProperties(Map<String, String> properties) {
    this.properties = properties;
    return this;
  }

  ActivitiesQuery withMentions(List<UserId> mentions) {
    this.mentions = mentions;
    return this;
  }

  AnnouncementsQuery? asAnnouncementsQuery() {
    switch (this.ids.type) {
      case CommunitiesEntityType.topic:
        return AnnouncementsQuery.inTopic(this.ids.ids.first);
      case CommunitiesEntityType.group:
        return AnnouncementsQuery.inGroup(this.ids.ids.first);
      case CommunitiesEntityType.app:
        return AnnouncementsQuery.timeline();
      case CommunitiesEntityType.user:
        return AnnouncementsQuery.forFeedOf(UserId.create(this.ids.ids.first));
    }
    return null;
  }

  Map<String, dynamic> toJSON() => {
        'author': author?.toJSON(),
        'ids': ids.toJSON(),
        'labels': labels,
        'mentions': mentions,
        'pollStatus': pollStatus,
        'properties': properties,
        'reactions': reactions,
        'reactionGroup': reactionGroup,
        'searchTerm': searchTerm,
        'tag': tag,
        'trending': trending,
      };
}
