import '../../common/codable.dart';
import '../../common/userid.dart';

class GroupsQuery extends Encodable {
  String? searchTerm;
  UserId? follower;
  UserId? member;
  bool trending = false;
  List<String> labels = [];
  Map<String, String> properties = {};

  GroupsQuery(String? searchTerm) {
    this.searchTerm = searchTerm;
  }

  static GroupsQuery find(String searchTerm) {
    return GroupsQuery(searchTerm);
  }

  static GroupsQuery all() {
    return GroupsQuery(null);
  }

  GroupsQuery followedBy(UserId id) {
    this.follower = id;
    return this;
  }

  GroupsQuery withMember(UserId id) {
    this.member = id;
    return this;
  }

  GroupsQuery onlyTrending(bool trending) {
    this.trending = trending;
    return this;
  }

  GroupsQuery withLabels(List<String> labels) {
    this.labels = labels;
    return this;
  }

  GroupsQuery withProperties(Map<String, String> properties) {
    this.properties = properties;
    return this;
  }

  Map<String, dynamic> toJSON() => {
        'followerId': follower?.toJSON(),
        'labels': labels,
        'memberId': member?.toJSON(),
        'properties': properties,
        'searchTerm': searchTerm,
        'trending': trending
      };
}
