import '../../common/codable.dart';
import '../../common/userid.dart';

class TopicsQuery extends Encodable {
  String? searchTerm;
  UserId? follower;
  bool trending = false;
  List<String> labels = [];
  Map<String, String> properties = {};

  TopicsQuery(String? searchTerm) {
    this.searchTerm = searchTerm;
  }

  static TopicsQuery find(String searchTerm) {
    return TopicsQuery(searchTerm);
  }

  static TopicsQuery all() {
    return TopicsQuery(null);
  }

  TopicsQuery followedBy(UserId id) {
    this.follower = id;
    return this;
  }

  TopicsQuery onlyTrending(bool trending) {
    this.trending = trending;
    return this;
  }

  TopicsQuery withLabels(List<String> labels) {
    this.labels = labels;
    return this;
  }

  TopicsQuery withProperties(Map<String, String> properties) {
    this.properties = properties;
    return this;
  }

  Map<String, dynamic> toJSON() => {
        'followerId': follower?.toJSON(),
        'labels': labels,
        'properties': properties,
        'searchTerm': searchTerm ?? '',
        'trending': trending
      };
}
